(* Proof Accelerator extraction (http://proofos.sf.net/accel/)
 * Copyright (C) 2005  The Regents of the University of California
 * Author: Adam Chlipala
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *)

(* Custom OCaml extraction optimizations *)

open Pp
open Names
open Miniml

type inductive_name = string
(* This names an (mutually) inductive type (nest).
 * The string should be in the format that Names.string_of_kn would generate for
 * that inductive type.
 *)

type inductive_info = {
    ciPrintAst : (ml_ast -> std_ppcmds) -> ml_ast -> std_ppcmds option;
      (* You have the option of specifying alternate printing for some
       * expressions of this type.  This is currently only called for
       * applications of one of this type's constructors.
       * The first argument can be used to pretty-print sub-expressions.
       *)
      ciPrintMatch : (identifier list -> std_ppcmds
	-> std_ppcmds * std_ppcmds) list;
      (* How to print a 'match' rule that uses this constructor, with one printer
       * function per constructor.  (If there are fewer printers than constructors,
       * the later constructors get normal treatment.)
       * Each function will be passed the variables standing for parameters,
       * as well as the pretty-printed (normal) body of the match rule.
       * It should return the pretty-printed pattern and body, respectively.
       *)
  }

val is_special_type : inductive_name -> bool
val special_type : inductive_name -> inductive_info
