(* Proof Accelerator library (http://proofos.sf.net/accel/)
 * Copyright (C) 2005  The Regents of the University of California
 * Author: Adam Chlipala
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *)

(** Implementing Coq's Bvector type conservatively with [int64] *)

type 'a bvector = int64

exception Overflow

let xO n = Int64.shift_left n 1
let xI n = Int64.add (Int64.shift_left n 1) Int64.one

let bit32 = Int64.shift_left Int64.one 32
let mask32 = Int64.sub bit32 Int64.one

let trunc n = Int64.logand n mask32

let plus n m = trunc (Int64.add n m)
let minus n m = trunc (Int64.sub n m)
let times n m = trunc (Int64.mul n m)

let inc len n = trunc (Int64.add n Int64.one)
let dec len n = trunc (Int64.sub n Int64.one)

let max_nat = Int64.of_int max_int

let bvec_to_nat len v = Big_int.big_int_of_string (Int64.to_string v)
let nat_to_bvec len n = Int64.of_string (Big_int.string_of_big_int n)

let bvec_eq_dec len (v1 : int64) v2 = v1 = v2

let bneg len v = trunc (Int64.lognot v)
let bVand len v1 v2 = Int64.logand v1 v2
let bVor len v1 v2 = Int64.logor v1 v2
