(* Proof Accelerator library (http://proofos.sf.net/accel/)
 * Copyright (C) 2005  The Regents of the University of California
 * Author: Adam Chlipala
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *)

(** Implementing Coq's nat type conservatively with [int] *)

exception Overflow

module H = Hashtbl

open Big_int

type nat = big_int

let two = big_int_of_int 2

let succ n = add_big_int n unit_big_int
let pred n = sub_big_int n unit_big_int

let plus = add_big_int

let minus n1 n2 =
  let r = sub_big_int n1 n2 in
  if lt_big_int r zero_big_int then
    zero_big_int
  else
    r

let times = mult_big_int

let eq_nat_dec = eq_big_int

let int_table = H.create 10

let of_int n =
  try
    H.find int_table n
  with Not_found ->
    let bn = big_int_of_int n in
    H.add int_table n bn;
    bn

let string_table = H.create 10

let of_string s =
  try
    H.find string_table s
  with Not_found ->
    let bn = big_int_of_string s in
    H.add string_table s bn;
    bn
