(* Copyright (c) 2005, Regents of the University of California
 * All rights reserved.
 *
 * Author: Adam Chlipala
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name of the University of California, Berkeley nor the names of
 *   its contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *)

(** Useful tactics related to bitvectors *)

Require Import Bool Bvector.

Require Import Asm.Util.Tactics.
Require Import Asm.Util.Vector.

(** Version of [Asm.Util.Vector.myVOtac] for [Bvector] *)
Ltac myVOtac := unfold Bvector in *; VOtac.

(** Version of [Asm.Util.Vector.myVSntac] for [Bvector] *)
Ltac myVSntac H x := unfold Bvector in *; VSntac H x.

(** Induction skeleton for proving goals of the form [forall (len : nat) (v : Bvector len), _] *)
Ltac vec_ind :=
  let H := fresh in let Htail := fresh in
    (intro len; induction len as [ | len IHlen ]; intro v;
      [myVOtac; try (simpl; intuition; fail)
	| myVSntac H v; destruct (Vhead v); simpl; myrewrite;
	  generalize (IHlen (Vtail v)); try (intuition; fail); try omega; intro Htail]).
