(* Copyright (c) 2005-2006, Regents of the University of California
 * All rights reserved.
 *
 * Author: Adam Chlipala
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name of the University of California, Berkeley nor the names of
 *   its contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *)

Set Implicit Arguments.

Require Import List TheoryList.

Require Import Asm.Util.Specif.
Require Import Asm.Util.Tactics.

(* Useful properties of lists *)

Lemma In_AllS : forall (T : Set) (f : T -> Prop) (ls : list T),
  (forall x, In x ls -> f x)
  -> AllS f ls.
  induction ls; intuition.
Qed.

Lemma In_dec : forall (T : Set) (eq_dec : forall (x y : T), {x = y} + {x <> y})
  (x : T) ls,
  {In x ls} + {~In x ls}.
  induction ls; simpl; auto.
  destruct (eq_dec a x); subst; auto.
  destruct IHls; intuition.
Qed.

Lemma AllS_dec : forall T f (ls : list T),
  {AllS (fun x => f x = true) ls} + {~AllS (fun x => f x = true) ls}.
  Hint Constructors AllS.
  induction ls; auto.
  destruct IHls; intuition.
  destrEq Hf (f a); auto.
  right.
  intuition.
  inversion H; congruence.
  right.
  intuition.
  inversion H; auto.
Qed.

Lemma AllS_pdec : forall (T : Set) (P : T -> Prop) (f : forall x : T, pred_option (P x)) (ls : list T),
  {AllS (fun x => popt_Prop (f x)) ls} + {~AllS (fun x => popt_Prop (f x)) ls}.
  Hint Constructors AllS.
  induction ls; auto.
  destruct IHls; intuition.
  destrEq Hf (f a); auto.
  left.
  apply allS_cons; intuition.
  red.
  rewrite H.
  trivial.
  right.
  intuition.
  inversion H; subst.
  rewrite Hf in H2.
  tauto.
  right.
  intuition.
  inversion H; auto.
Qed.

Lemma AllS_dec_option : forall T f (ls : list T),
  pred_option (AllS (fun x => f x = true) ls).
  intros.
  destruct (AllS_dec f ls).
  constructor; trivial.
  apply PNone.
Qed.

Lemma AllS_pdec_option : forall (T : Set) (P : T -> Prop) (f : forall x : T, pred_option (P x)) (ls : list T),
  pred_option (AllS (fun x => popt_Prop (f x)) ls).
  intros.
  destruct (AllS_pdec P f ls).
  constructor; trivial.
  apply PNone.
Qed.

Lemma AllS_In : forall T f (ls : list T),
  AllS f ls
  -> forall x, In x ls
    -> f x.
  induction ls; intuition.
  inversion H0.
  inversion H; subst.
  inversion H0.
  subst; trivial.
  auto.
Qed.

Lemma AllS_imp : forall (T : Set) (p1 p2 : T -> Prop),
  (forall x, p1 x -> p2 x)
  -> forall ls,
    AllS p1 ls -> AllS p2 ls.
  Hint Constructors AllS.
  induction ls; auto.
  intros Hall; inversion Hall; subst; auto.
Qed.

Lemma In_map : forall (T S : Set) (f : T -> S) ls x,
  In x (map f ls)
  -> exists y, In y ls /\ x = f y.
  induction ls; intuition; simpl in *; intuition; subst; eauto.
  destruct (IHls _ H0).
  intuition; eauto.
Qed.

Lemma In_map' : forall (T S : Set) (f : T -> S) ls x,
  (exists y, In y ls /\ x = f y)
  -> In x (map f ls).
  induction ls; simpl; firstorder; subst; firstorder.
Qed.

Section iter.
  Variable T : Set.
  Variable f : T -> unit.

  Fixpoint iter' (v : unit) (ls : list T) {struct ls} : unit :=
    match ls with
      | nil => tt
      | cons x ls => iter' (f x) ls
    end.

  Definition iter := iter' tt.
End iter.
