(* Copyright (c) 2005-2006, Regents of the University of California
 * All rights reserved.
 *
 * Author: Adam Chlipala
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name of the University of California, Berkeley nor the names of
 *   its contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *)

(** Monads for state and failure handling *)

Set Implicit Arguments.

Require Import Asm.Util.Specif.

Section sig_option.
  Variable T : Set.
  Variable P : T -> Prop.

  Section sig_option_sig_option.
    Variable T' : Set.
    Variable P' : T' -> Prop.

    Definition try_sig_option_sig_option (o : sig_option P) (f : forall x, P x -> sig_option P') :=
      match o with
	| SNone => SNone _
	| SSome x pf => f x pf
      end.
  End sig_option_sig_option.

  Section sig_option_pred_option.
    Variable P' : Prop.

    Definition try_sig_option_pred_option (o : sig_option P) (f : forall x, P x -> pred_option P') :=
      match o with
	| SNone => PNone _
	| SSome x pf => f x pf
      end.
  End sig_option_pred_option.
End sig_option.

Section pred_option.
  Variable P : Prop.

  Section pred_option_sig_option.
    Variable T : Set.
    Variable P' : T -> Prop.

    Definition try_pred_option_sig_option (o : pred_option P) (f : P -> sig_option P') :=
      match o with
	| PNone => SNone _
	| PSome pf => f pf
      end.
  End pred_option_sig_option.

  Section pred_option_pred_option.
    Variable P' : Prop.

    Definition try_pred_option_pred_option (o : pred_option P) (f : P -> pred_option P') :=
      match o with
	| PNone => PNone _
	| PSome pf => f pf
      end.
  End pred_option_pred_option.
End pred_option.

Section sig.
  Variable T : Set.
  Variable P : T -> Prop.

  Section sig_anything.
    Variable T' : Set.

    Definition try_sig_anything (o : sig P) (f : forall x, P x -> T') :=
      match o with
	| exist x pf => f x pf
      end.
  End sig_anything.
End sig.

Notation "x ::: pf <- o ; f" := (try_sig_anything o (fun x => fun pf => f))
  (at level 60, right associativity).
Notation "x ::: pf <- o ;; f" := (try_sig_anything o (fun x => fun pf => f))
  (at level 60, right associativity).

Notation "x <-- o ; f" := (try_pred_option_sig_option o (fun x => f))
  (at level 60, right associativity).
Notation "x <-- o ;; f" := (try_pred_option_pred_option o (fun x => f))
  (at level 60, right associativity).

Notation "x :: pf <- o ; f" := (try_sig_option_sig_option o (fun x => fun pf => f))
  (at level 60, right associativity).
Notation "x :: pf <- o ;; f" := (try_sig_option_pred_option o (fun x => fun pf => f))
  (at level 60, right associativity).
