(* Copyright (c) 2005-2006, Regents of the University of California
 * All rights reserved.
 *
 * Author: Adam Chlipala
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name of the University of California, Berkeley nor the names of
 *   its contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *)

(** Facts about memories *)

Require Import Asm.Util.ArithUtil.
Require Import Asm.Util.Tactics.

Require Import Asm.Bitvector.Bitvector.

Require Import Asm.X86.Defs.

Set Implicit Arguments.

(** * Inverting byte-sized memory operations *)

(*Lemma invert_set_mem' : forall m addr (v : int8),
  Int32Map.get addr (Int32Map.set addr v m) = v.
  intros; apply Int32Map.gss.
Qed.*)

Lemma invert_set_mem' : forall m addr (v : int8),
  v = Int32Map.get addr (Int32Map.set addr v m).
  intros; symmetry; apply Int32Map.gss.
Qed.

(*Lemma invert_set_mem_neq : forall st addr addr' v,
  addr <> addr'
  -> get_mem (set_mem st addr v) addr' = get_mem st addr'.
  intros; unfold get_mem, set_mem; simpl; apply Int32Map.gso; auto.
Qed.*)

Lemma eq_get_mem_neq : forall addr addr' x m (v : int8),
  addr <> addr'
  -> x = Int32Map.get addr' m
  -> x = Int32Map.get addr' (Int32Map.set addr v m).
  intros; subst; symmetry; apply Int32Map.gso; auto.
Qed.

Lemma neq_sym : forall (n1 n2 : int32),
  n1 <> n2
  -> n2 <> n1.
  intros; auto.
Qed.

Hint Resolve Int32Map.gss invert_set_mem' Int32Map.gso eq_get_mem_neq : x86.
Hint Resolve binc_neq binc2_neq binc3_neq binc_neq' binc2_neq' binc3_neq' neq_sym : x86.
Hint Unfold set_mem32' : x86.


(** * Inverting word-sized memory operations *)

Ltac binc_binc :=
  let H := fresh in
    (intro H; try exact (binc_neq' H); try exact (binc2_neq' H); try exact (binc3_neq' H)).

Hint Extern 1 (?X1 <> ?X2) => binc_binc : x86.

Lemma invert_set_mem32' : forall m addr v,
  get_mem32' (set_mem32' m addr v) addr = v.
  intros.
  unfold get_mem32'.
  replace (Int32Map.get addr (set_mem32' m addr v)) with (byte1 v);
    [idtac | unfold set_mem32'; auto with x86].
  replace (Int32Map.get (binc addr) (set_mem32' m addr v)) with (byte2 v);
    [idtac | unfold set_mem32'; auto with x86].
  replace (Int32Map.get (binc (binc addr)) (set_mem32' m addr v)) with (byte3 v);
    [idtac | unfold set_mem32'; auto with x86].
  replace (Int32Map.get (binc (binc (binc addr))) (set_mem32' m addr v)) with (byte4 v);
    [idtac | unfold set_mem32'; auto with x86].
  rewrite <- (split32_sound v).
  apply (btn_invert v).
Qed.

Lemma invert_set_mem32 : forall st addr v,
  get_mem32 (set_mem32 st addr v) addr = v.
  intros; unfold get_mem32, set_mem32; simpl; apply invert_set_mem32'.
Qed.

Inductive apart : int32 -> int32 -> Prop :=
  | apart_lower : forall a a',
    bvec_to_nat a' + 4 <= bvec_to_nat a
    -> apart a a'
  | apart_higher : forall a a',
    bvec_to_nat a' >= bvec_to_nat a + 4
    -> apart a a'.

Lemma apart_neq : forall n m,
  apart n m
  -> n <> m.
  intros.
  inversion H; subst; 
    [assert (bvec_to_nat n <> bvec_to_nat m); [omega | congruence]
      | assert (bvec_to_nat n <> bvec_to_nat m); [omega | congruence]].
Qed.

Lemma apart_binc_neq : forall n m,
  belowMax n 1
  -> apart n m
  -> binc n <> m.
  intros; intro.
  subst.
  inversion H0; subst;
    rewrite binc_btn_belowMax in H1; trivial; omega.
Qed.

Lemma apart_binc2_neq : forall n m,
  belowMax n 2
  -> apart n m
  -> binc2 n <> m.
  intros; intro.
  subst.
  inversion H0; subst;
    rewrite binc2_btn_belowMax in H1; trivial; omega.
Qed.

Lemma apart_binc3_neq : forall n m,
  belowMax n 3
  -> apart n m
  -> binc3 n <> m.
  intros; intro.
  subst.
  inversion H0; subst;
    rewrite binc3_btn_belowMax in H1; trivial; omega.
Qed.

Definition not_too_high (n : int32) := belowMax n 3.

Lemma invert_set_mem32'_neq' : forall m addr addr' v,
  not_too_high addr
  -> apart addr addr'
  -> Int32Map.get addr' (set_mem32' m addr v) = Int32Map.get addr' m.
  intros m addr addr' v Hnth Hapart.
  unfold set_mem32'.
  unfold not_too_high in Hnth.
  repeat (rewrite Int32Map.gso); trivial; apply neq_sym;
    [apply apart_neq | apply apart_binc_neq | apply apart_binc2_neq | apply apart_binc3_neq];
    myeauto.
Qed.

Lemma apart_sym : forall n m,
  apart n m
  -> apart m n.
  Hint Constructors apart.
  intuition.
  inversion H; auto.
Qed.

Ltac btn_conv :=
  match goal with
    | [ |- ?X <> ?Y ] => (assert (bvec_to_nat X <> bvec_to_nat Y); try congruence)
  end.

Ltac prove_bincs_neq :=
  btn_conv;
  repeat ((rewrite binc_btn_belowMax || rewrite binc2_btn_belowMax || rewrite binc3_btn_belowMax);
    myeauto);
    inversion Hapart; try omega.

Lemma invert_set_mem32'_binc_neq' : forall m addr addr' v,
  not_too_high addr
  -> not_too_high addr'
  -> apart addr addr'
  -> Int32Map.get (binc addr') (set_mem32' m addr v) = Int32Map.get (binc addr') m.
  intros m addr addr' v Hnth Hnth' Hapart.
  unfold set_mem32'.
  unfold not_too_high in Hnth.
  unfold not_too_high in Hnth'.

  repeat (rewrite Int32Map.gso); trivial; prove_bincs_neq.
Qed.

Lemma invert_set_mem32'_binc2_neq' : forall m addr addr' v,
  not_too_high addr
  -> not_too_high addr'
  -> apart addr addr'
  -> Int32Map.get (binc (binc addr')) (set_mem32' m addr v) = Int32Map.get (binc (binc addr')) m.
  intros m addr addr' v Hnth Hnth' Hapart.
  unfold set_mem32'.
  unfold not_too_high in Hnth.
  unfold not_too_high in Hnth'.

  repeat (rewrite Int32Map.gso); trivial; prove_bincs_neq.
Qed.

Lemma invert_set_mem32'_binc3_neq' : forall m addr addr' v,
  not_too_high addr
  -> not_too_high addr'
  -> apart addr addr'
  -> Int32Map.get (binc (binc (binc addr'))) (set_mem32' m addr v) = Int32Map.get (binc (binc (binc addr'))) m.
  intros m addr addr' v Hnth Hnth' Hapart.
  unfold set_mem32'.
  unfold not_too_high in Hnth.
  unfold not_too_high in Hnth'.

  repeat (rewrite Int32Map.gso); trivial; prove_bincs_neq.
Qed.

Theorem invert_set_mem32'_neq : forall m addr addr' v,
  not_too_high addr
  -> not_too_high addr'
  -> apart addr addr'
  -> get_mem32' (set_mem32' m addr v) addr' = get_mem32' m addr'.
  intros m addr addr' v Hnth Hnth' Hapart.
  unfold get_mem32'.
  generalize (pow2 8); generalize (pow2 16); generalize (pow2 24).
  intros.
  rewrite (invert_set_mem32'_neq' m (addr := addr) (addr' := addr')); trivial.
  rewrite (invert_set_mem32'_binc_neq' m (addr := addr) (addr' := addr')); trivial.
  rewrite (invert_set_mem32'_binc2_neq' m (addr := addr) (addr' := addr')); trivial.
  rewrite (invert_set_mem32'_binc3_neq' m (addr := addr) (addr' := addr')); trivial.
Qed.

Theorem invert_set_mem32_neq : forall st addr addr' v,
  not_too_high addr
  -> not_too_high addr'
  -> apart addr addr'
  -> get_mem32 (set_mem32 st addr v) addr' = get_mem32 st addr'.
  intros.
  unfold get_mem32, set_mem32; simpl; apply invert_set_mem32'_neq; trivial.
Qed.

