(* Copyright (c) 2005, Regents of the University of California
 * All rights reserved.
 *
 * Author: Adam Chlipala
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name of the University of California, Berkeley nor the names of
 *   its contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *)

(** Basic definitions *)

Require Import Arith.

Require Import Asm.Util.Tactics.

Require Import Asm.Bitvector.Bitvector.

Require Import Asm.X86.X86.

Require Import ProofOS.Trusted.Safety.
Require Import ProofOS.Trusted.Env.

Require Import ProofOS.AbsInt.Bounded.

Require Import ProofOS.Kernel.Defs.
Require Import ProofOS.Kernel.Init.
Require Import ProofOS.Kernel.Step.
Require Import ProofOS.Kernel.Types.
Require Import ProofOS.Kernel.Abs.

Module M86 <: MACHINE.
  Definition mstate := state.
  Definition minitState := initState.
  Definition minstr := instr.
  Definition mget_instr := get_instr.
  Definition mexec := exec.
End M86.

Module Abs <: ABSTRACTION.
  Module Mac := M86.

  Definition mastate := astate.
  Definition mastate_eq_dec := astate_eq_dec.

  Definition mcontext := context.

  Definition mconc := conc'.
  Definition maget_instr abs :=
    match aPc abs with
      | apcConst pc => Some (decode_instr initialMem pc)
      | _ => None
    end.
  Theorem maget_instr_sound : forall abs st ctx v,
    conc' abs st ctx
    -> maget_instr abs = Some v
    -> v = Mac.mget_instr st.
    intros abs st ctx v Hconc Hv.
    inversion Hconc.
    unfold maget_instr in Hv.
    unfold Mac.mget_instr.
    unfold get_instr.
    red in concPc.
    destruct (aPc abs); subst.
    rewrite (@decode_same (stMem st) initialMem (stPc st)).
    congruence.
    red in concCodeMem.
    red.
    intros.
    apply concCodeMem.
    red in concPcRange.
    omega.
    discriminate.
  Qed.
  
  Definition minit := init.
  Definition mstep := step.
End Abs.

Module C := Checker(Abs).
