(* Copyright (c) 2005, Regents of the University of California
 * All rights reserved.
 *
 * Author: Adam Chlipala
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name of the University of California, Berkeley nor the names of
 *   its contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *)

(** Program verification hints, packaged as metadata *)

Require Import List.

Require Import Asm.Bitvector.Bitvector.

Require Import Asm.X86.X86.

Require Import ProofOS.Kernel.Defs.


(** * Cutpoints *)

(** An assertion of an invariant *)
Record cutpoint_info : Set := {
  ciPc : apc;                 (** The program counter *)
  ciStackSize : nat;          (** Minimum stack size *)
  ciRegs : list (reg32 * ty); (** Types of registers *)
  ciSlots : list (nat * ty)   (** Types of stack slots *)
}.

Parameter cutpoints : list (list cutpoint_info * cutpoint_info).
(** The cutpoints asserted for the current program.
  Each entry contains a list of assumed-safe states and the actual entry state.
  *)


(** * Function signatures *)

Record function_info : Set := {
  fiRet : ty;                 (** Return type *)
  fiArgs : list ty            (** How many arguments? *)
}.

Parameter functions : int32 -> option function_info.
