(* Copyright (c) 2005, Regents of the University of California
 * All rights reserved.
 *
 * Author: Adam Chlipala
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name of the University of California, Berkeley nor the names of
 *   its contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *)

(** Miscellaneous definitions and proofs *)

Require Import Arith List.

Require Import Asm.Util.ArithUtil.
Require Import Asm.Util.Mod.
Require Import Asm.Util.Tactics.

Require Import Asm.Bitvector.Bitvector.

Require Import Asm.X86.X86.

Require Import ProofOS.Trusted.Env.

Require Import ProofOS.Kernel.Defs.

Definition int32_In_dec : forall (n : int32) ls,
  {In n ls} + {~In n ls}.
  induction ls.
  right; tauto.
  destruct (int32_eq_dec n a); subst; intuition.
  right.
  intuition.
  simpl in H.
  intuition.
Qed.

Definition pcInRange_dec : forall pc, {pcInRange pc} + {~pcInRange pc}.
  intro.
  destruct (le_lt_dec (bvec_to_nat initialPc) (bvec_to_nat pc));
    destruct (le_lt_dec (bvec_to_nat pc) (bvec_to_nat initialPc + actualCodeLen));
      [left; red; intuition
	| right; unfold pcInRange; intuition
	| right; unfold pcInRange; intuition
	| right; unfold pcInRange; intuition].
Defined.

Lemma apart_dec : forall n m,
  {apart n m} + {~apart n m}.
  intros.
  destruct (le_lt_dec (bvec_to_nat m + 4) (bvec_to_nat n));
    [left; apply apart_lower; trivial
      | destruct (le_lt_dec (bvec_to_nat n + 4) (bvec_to_nat m));
	[left; apply apart_higher; omega
	  | right; intro; inversion H; omega]].
Qed.

Definition nat_apart n m :=
  m + 4 <= n \/ n + 4 <= m.

Lemma nat_apart_dec : forall n m,
  {nat_apart n m} + {~nat_apart n m}.
  intros.
  destruct (le_lt_dec (m + 4) n);
    [left; red; tauto
      | destruct (le_lt_dec (n + 4) m);
	[left; red; tauto
	  | right; unfold nat_apart; omega]].
Qed.

Lemma arith_op_eq_dec : forall (a1 a2 : arith_op), {a1 = a2} + {a1 <> a2}.
  intros.
  decide equality a1 a2.
Defined.

Lemma ntb_invert_eqMod : forall len n1 n2 n k,
  bvec_to_nat (nat_to_bvec len n) <= n1
  -> n <= k * pow2 len
  -> eqMod (pow2 len) (n1 + k * pow2 len - n) n2
  -> eqMod (pow2 len) (n1 - bvec_to_nat (nat_to_bvec len n)) n2.
  intros len n1 n2 n k Hle Hpow2 Hmod.
  apply eqMod_trans with (n1 + k * pow2 len - n); trivial;
    apply eqMod_minus; myauto;
      [apply eqMod_trans with (n1 + pow2 len);
	[apply eqMod_add_mod
	  | apply eqMod_plus;
	    [apply eqMod_refl
	      | apply eqMod_mod_times]]
	| apply ntb_invert_mod
	| omega].
Qed.
