(* Copyright (c) 2005, Regents of the University of California
 * All rights reserved.
 *
 * Author: Adam Chlipala
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name of the University of California, Berkeley nor the names of
 *   its contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *)

(** Operations on types *)

Require Import Arith.

Require Import Asm.Bitvector.Bitvector.

Require Import Asm.Util.Tactics.
Require Import Asm.Util.Specif.

Require Import ProofOS.Kernel.Defs.

Hint Constructors hasType : asm.

Set Implicit Arguments.

Definition ty_eq_dec : forall (t1 t2 : ty), {t1 = t2} + {t1 <> t2}.
  intros t1 t2.
  decide equality t1 t2.
  unfold int32 in *; apply bvec_eq_dec.
  apply eq_nat_dec.
Qed.

Definition subType t1 t2 :=
  match t2 with
    | Top => true
    | _ => sum_to_bool (ty_eq_dec t1 t2)
  end.

Definition subReg abs1 abs2 r :=
  subType (aget_reg abs1 r) (aget_reg abs2 r).

Theorem subType_refl : forall t, subType t t = true.
  intro.
  unfold subType.
  destruct (ty_eq_dec t t); simpl; destruct t; tauto.
Qed.

Theorem subType_sound : forall t1 t2,
  subType t1 t2 = true
  -> forall ctx v, hasType ctx v t1
    -> hasType ctx v t2.
  intros t1 t2 Hsub ctx v Ht1.
  destruct t2; try myeauto;
    match goal with
      | [ |- hasType ctx v ?T ] =>
	simpl in Hsub; destruct (ty_eq_dec t1 T) as [e | e];
	  [rewrite e in Ht1; trivial | discriminate]
    end.
Qed.
