(* Copyright (c) 2005, Regents of the University of California
 * All rights reserved.
 *
 * Author: Adam Chlipala
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name of the University of California, Berkeley nor the names of
 *   its contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *)

(** A stateless safety policy functor *)

Set Implicit Arguments.

Require Import Asm.Bitvector.Bitvector.

Require Import ProofOS.Machine.Machine.

Module Type STATELESS.
  Declare Module Mac : MACHINE.

  Parameter isSafe : Mac.minstr -> Mac.mstate -> Prop.
End STATELESS.

Inductive stateless_mexec (minstr mstate : Set)
  (mexec : minstr * int32 -> mstate -> mstate -> Prop)
  (isSafe : minstr -> mstate -> Prop)
  : minstr * int32 -> mstate -> mstate -> Prop :=
  | stateless_mexec_rule : forall ins nextPc stIn stOut,
    isSafe ins stIn
    -> mexec (ins, nextPc) stIn stOut
    -> stateless_mexec mexec isSafe (ins, nextPc) stIn stOut.

Module Stateless (Input : STATELESS) <: MACHINE.
  Import Input.

  Definition mstate := Mac.mstate.
  Definition minstr := Mac.minstr.

  Definition minitState := Mac.minitState.
  Definition mget_instr := Mac.mget_instr.

  Definition mexec := stateless_mexec Mac.mexec isSafe.
End Stateless.
