(* Copyright (c) 2005-2006, Regents of the University of California
 * All rights reserved.
 *
 * Author: Adam Chlipala
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name of the University of California, Berkeley nor the names of
 *   its contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *)

(** Abstract machine safety *)

Require Import List.
Require Import Asm.Bitvector.Bitvector.

Set Implicit Arguments.

Module Type MACHINE.
  Parameter mstate : Set.
  Parameter minitState : mstate -> Prop.

  Parameter minstr : Set.
  Parameter mget_instr : mstate -> minstr * int32.
  Parameter mexec : minstr * int32 -> mstate -> mstate -> Prop.
End MACHINE.

Definition cstep (mstate minstr : Set)
  (mexec : (minstr * int32) -> mstate -> mstate -> Prop)
  mget_instr st := mexec (mget_instr st) st.

CoInductive safe (mstate : Set) (cstep : mstate -> mstate -> Prop) : mstate -> Prop :=
    | safe_rule : forall st st',
      cstep st st'
      -> (forall st'' : mstate, cstep st st'' -> safe cstep st'')
      -> safe cstep st.

Module Safety (Mac : MACHINE).
  Import Mac.
  
  Definition cstep := cstep mexec mget_instr.

  Definition safe := safe cstep.
End Safety.
