(* Copyright (c) 2005, Regents of the University of California
 * All rights reserved.
 *
 * Author: Adam Chlipala
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name of the University of California, Berkeley nor the names of
 *   its contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *)

(** Support code for FirstOrder verifiers *)

open AsmUtil
open X86Types
open Ver_types
open FirstOrder_types


(** {6 Type translation} *)

val word_type_to_ty : 'a word_type -> 'a ty

val ty_to_word_type : 'a ty -> 'a word_type


(** {6 Memory layout parameters} *)

val initialPc : bool CoqBvector.bvector
(** Beginning of this module's code region *)

val regions_loc : int
(** Beginning of our regions of memory, presumably after the area reserved for
   the boot-loader *)

val stackSize : CoqNat.nat
(** Lower bound on the amount of stack space available *)

val stackStart : bool CoqBvector.bvector
(** Lowest address that is considered part of the stack region *)

val actualCodeLen : CoqNat.nat
(** How much space is actually used by the code segment *)


(** {6 Record constructors} *)

val build_cutpoint_info : apc
  -> CoqNat.nat
    -> (reg32, 'a ty) prod coqlist
      -> (CoqNat.nat, 'a ty) prod coqlist
	-> 'b
	  -> ('a, 'b) cutpoint_info

val build_function_info : 'a ty -> 'a ty coqlist -> 'a function_info


(** {6 Parsing the binary image} *)

module Parse : functor (Param : FIRSTORDER)
  -> FIRSTORDER_PARSE with type typ = Param.typ and type astat = Param.astat
