(* Copyright (c) 2005, Regents of the University of California
 * All rights reserved.
 *
 * Author: Adam Chlipala
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name of the University of California, Berkeley nor the names of
 *   its contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *)

(** Environment for IntList verifier *)

open AsmUtil
open Format
open X86Print

type typ =
    Int
  | List of typ
  | NeList of typ

let rec read_typ bits =
  let kind, bits' = read_uint bits 32 in
  match kind with
    0 -> Int, bits'
  | 1 ->
      let t, bits' = read_typ bits' in
      List t, bits'
  | 2 ->
      let t, bits' = read_typ bits' in
      NeList t, bits'
  | _ -> failwith "Invalid IntList type to deserialize"

let rec pp_typ fmt = function
    Int -> fprintf fmt "Int"
  | List t -> fprintf fmt "List(%a)" pp_typ t
  | NeList t -> fprintf fmt "NeList(%a)" pp_typ t

let pp_astat fmt ropt =
  fprintf fmt "CmpOp = ";
  match ropt with
    None -> fprintf fmt "None@\n"
  | Some r -> fprintf fmt "%a@\n" pp_genop32 r

let read_astat bits = None, bits

