(* Copyright (c) 2005, Regents of the University of California
 * All rights reserved.
 *
 * Author: Adam Chlipala
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name of the University of California, Berkeley nor the names of
 *   its contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *)

(** Support code for the extracted clunky Kernel verifier *)

open AsmUtil
open X86Types
open Ver_types
open Kernel_types


(** {6 Memory layout parameters} *)

val regions_loc : int
(** Beginning of our regions of memory, presumably after the area reserved for
   the boot-loader *)

val cutpoints_loc : int
(** Beginning of cutpoint table *)

val stackSize : CoqNat.nat
(** Lower bound on the amount of stack space available *)

val stackStart : bool CoqBvector.bvector
(** Lowest address that is considered part of the stack region *)


(** {6 Parsing the binary image} *)

val actualCodeLen : CoqNat.nat
(** Actual number of bytes occupied by the code region *)

val cutpoints : (cutpoint_info coqlist, cutpoint_info) prod coqlist
(** All program point invariants *)

val functions : bool CoqBvector.bvector -> function_info option
(** Look for information on a function that begins at a particular address *)


(** {6 Pretty-printing} *)

val pp_ty : ty X86Print.printer
