(* Copyright (c) 2005, Regents of the University of California
 * All rights reserved.
 *
 * Author: Adam Chlipala
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name of the University of California, Berkeley nor the names of
 *   its contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *)

(** Support types for the extracted clunky Kernel verifier *)

open X86Types
open Ver_types


(** {6 Abstract states} *)

(** Abstract types of word-sized values *)
type ty =
  | Top                              (** Any value *)
  | Const of bool CoqBvector.bvector (** A known constant value *)
  | Stack of CoqNat.nat              (** Pointer to a constant stack slot *)
  | RetPtr                           (** Return pointer *)
  | OldEbp                           (** Saved value of EBP *)

(** Abstract program counters *)
type apc =
    ApcConst of bool CoqBvector.bvector (** Known program counter *)
  | ApcRet                              (** Return pointer *)


(** {6 Metadata} *)

(** Program point invariant *)
type cutpoint_info = {
    ciPc : apc;                             (** Program counter *)
    ciStackSize : CoqNat.nat;               (** Lower bound on available
					       stack space *)
    ciRegs : (reg32, ty) prod coqlist;      (** Register types *)
    ciSlots : (CoqNat.nat, ty) prod coqlist (** Stack slot types *)
  }

(** Function information *)
type function_info = {
    fiRet : ty;         (** Return type *)
    fiArgs : ty coqlist (** Argument types *)
  }
