Require Import ZArith BinInt Bool Bvector List.
Require Import Asm.Bitvector.Bitvector.
Require Import Asm.X86.X86.
Require Import Asm.Util.Specif.

Extract Inductive unit => "unit" [ "()" ].
Extract Inductive bool => "bool" [ "true" "false" ].
Extract Inductive sumbool => "bool" [ "true" "false" ].

Extract Inductive option => "option" [ "Some" "None" ].
Extract Inductive pred_option => "bool" [ "true" "false" ].
Extract Inductive positive => "int" [ "BadPositiveI" "BadPositiveO" "BadPositiveH" ].
Extract Inductive prod => "prod" [ "Pair" ].
Extract Inductive list => "coqlist" [ "Nil" "Cons" ].

Extract Inductive reg8 => "reg8" [ "AL" "CL" "DL" "BL" "AH" "CH" "DH" "BH" ].
Extract Inductive reg32 => "reg32" [ "EAX" "ECX" "EDX" "EBX" "ESP" "EBP" "ESI" "EDI" ].
Extract Inductive condition => "condition" [ "O" "B" "Z" "BE" "S" "P" "L" "LE" ].
Extract Inductive scale => "scale" [ "Scale1" "Scale2" "Scale4" "Scale8" ].
Extract Inductive genop => "genop" [ "Imm" "Reg" "Address" ].
Extract Inductive arith_op => "arith_op" [ "Add" "And" "Or" "Sub" ].
Extract Inductive shift_op => "shift_op" [ "Shl" "Shr" ].
Extract Inductive instr => "instr" [ "Arith" "Arithb" "Call" "Cmp" "Inc" "Jcc" "Jmp" "Lea" "Leave" "Mov" "Movb" "Pop" "Push" "Ret" "Shift" ].

Extract Inductive nat => "CoqNat.nat" [ "BadNatO" "BadNatS" ].
Extract Constant plus => "CoqNat.plus".
Extract Constant minus => "CoqNat.minus".
Extract Constant mult => "CoqNat.times".
Extract Constant eq_nat_dec => "CoqNat.eq_nat_dec".
Extract Constant lt_eq_lt_dec => "(fun n m -> if Big_int.eq_big_int n m then Inleft false else if Big_int.lt_big_int n m then Inleft true else Inright)".
Extract Constant le_lt_dec => "Big_int.le_big_int".
Extract Constant le_gt_dec => "Big_int.le_big_int".

Import BinInt.

Extract Inductive Z => "CoqZ.Z" [ "BadZ0" "BadZpos" "BadZneg" ].
Extract Constant Zplus => "CoqZ.plus".
Extract Constant Zminus => "CoqZ.minus".
Extract Constant Zmult => "CoqZ.times".

Extract Inductive vector => "CoqBvector.bvector" [ "BadVnil" "BadVcons" ].
Extract Constant int32 => "int64".
Extract Constant badd32 => "CoqBvector.plus".
Extract Constant bsub32 => "CoqBvector.minus".
Extract Constant binc => "CoqBvector.inc".
Extract Constant bdec => "CoqBvector.dec".
Extract Constant bvec_eq_dec => "CoqBvector.bvec_eq_dec".
Extract Constant bvec_to_nat => "CoqBvector.bvec_to_nat".
Extract Constant nat_to_bvec => "CoqBvector.nat_to_bvec".
Extract Constant Bneg => "CoqBvector.bneg".
Extract Constant BVand => "CoqBvector.bVand".
Extract Constant BVor => "CoqBvector.bVor".
Extract Constant bzero32 => "Int64.zero".

Extract Constant Vunary => "0".
Extract Constant Vbinary => "0".

Extract Constant initialMem => "buildMap (Int32Map.empty Int64.zero) Int32Map.upd".
Extract Constant decode_instr => "(fun mem addr -> if mem == initialMem then get_instr_from_bits addr else get_instr_failed ())".
