(* Copyright (c) 2005, Regents of the University of California
 * All rights reserved.
 *
 * Author: Adam Chlipala
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name of the University of California, Berkeley nor the names of
 *   its contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *)

(** Functionality used by all extracted verifiers *)

open X86Types
open Ver_types


(** {6 Finite maps} *)

module TotalMap : functor (T : ORDERED_SET) -> TOTAL_MAP with type key = T.t


(** {6 File information} *)

val binaryBits : AsmUtil.bits
(** The contents of the file being analyzed *)

val initialPc : bool CoqBvector.bvector
val initialPc_int : int
(** The address of the beginning of [binaryBits] *)

val buildMap : 'a -> ('a -> int64 -> int64 -> 'a) -> 'a
(** Used to create [initialMem]; it's a fold function that calls the function
   parameter with every address/value pair in [binaryBits]. *)

val get_instr_from_bits : int64 -> (instr, int64) prod
(** Read the instruction found at a given address, returning that instruction
   and the address of the instruction found immediately afterward. *)

val get_instr_failed : unit -> 'a
(** Abort with an error message saying that something went wrong in decoding an
   instruction. *)


(** {6 Support for extracted standard Coq types} *)

val coq_length : 'a coqlist -> int


(** {6 Pretty-printing} *)

val pp_print_big_int : Big_int.big_int X86Print.printer


