(* Copyright (c) 2005, Regents of the University of California
 * All rights reserved.
 *
 * Author: Adam Chlipala
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name of the University of California, Berkeley nor the names of
 *   its contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *)

(** Types used by all extracted verifiers *)

(** Duplications of standard Coq types *)

type 'a coqlist =
  | Nil
  | Cons of 'a * 'a coqlist

type 'a eq_dec = unit


(** Finite maps *)

module type ORDERED_SET = sig
  type t
  val eq_dec : t -> t -> bool
  val lt_dec : t -> t -> bool
end

module type TOTAL_MAP = sig
  type key
  type 'a map
  val empty : 'a -> 'a map
  val sel : 'a map -> key -> 'a
  val upd : 'a map -> key -> 'a -> 'a map
  val eq_dec : ('a -> 'a -> bool) -> 'a map -> 'a map -> bool
  val subMap : ('a -> 'a -> bool) -> 'a map -> 'a map -> bool
  val iter : (key -> 'a -> unit) -> 'a map -> unit
  val print :
      (Format.formatter -> key -> unit)
    -> (Format.formatter -> 'a -> unit)
      -> Format.formatter -> 'a map -> unit
  val mapMap : ('a -> 'b) -> 'a map -> 'b map
end
