(* Copyright (c) 2006, Regents of the University of California
 * All rights reserved.
 *
 * Author: Adam Chlipala
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name of the University of California, Berkeley nor the names of
 *   its contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *)

(** Functionality used by all extracted verifiers *)

open X86Types
open Ver_types

(** {6 Instruction decoding} *)

val decode_instr : 'a -> int64 -> (X86Types.instr, int64) X86Types.prod 
(** "Cheating" instruction decoder function.  It assumes that you only pass it
    the initial memory. *)


(** {6 Memory layout parameters}  *)

val codeStart : bool CoqBvector.bvector
val codeLength : CoqNat.nat

val stackStart : bool CoqBvector.bvector
val stackLength : CoqNat.nat

val heapStart : bool CoqBvector.bvector
val heapLength : CoqNat.nat


(** {6 Printing abstract states} *)

val print_big_int : Big_int.big_int X86Print.printer

val print_top : unit -> unit
val print_int32_option : bool CoqBvector.bvector option -> unit
val print_nostate : unit -> unit

val print_salReg : Ver_types.salReg -> unit
val print_salExp : Ver_types.salExp -> unit
val print_salExp_option : Ver_types.salExp option -> unit
val print_weakState : (((salReg, salExp option) prod -> unit) -> unit) -> unit
val print_stackTy : ('ty -> unit) -> 'ty Ver_types.stackTy -> unit
val print_stateDescription : ('ty -> unit) -> ('mastate -> unit)
  -> apc -> 'ty
    -> (((Ver_types.salReg, 'ty) X86Types.prod -> unit) -> unit)
      -> 'mastate -> unit
val print_stackState : ('ty -> unit) -> ('mastate -> unit)
  -> CoqNat.nat -> 'ty Ver_types.stackTy
    -> (((CoqNat.nat, 'ty Ver_types.stackTy) X86Types.prod -> unit) -> unit)
      -> 'mastate -> unit
val print_condState : ('a -> unit) -> (((flag, ((arith_op, salExp)
						  prod, salExp) prod option) prod -> unit) -> unit)
  -> 'a -> unit

(** {6 Parsing metadata} *)

module Parser : functor (Param : METADATA)
  -> METADATA_PARSE with type 'a coq_list = 'a Param.coq_list
  and type stateDescription = Param.stateDescription
  and type function_info = Param.function_info
