/* Copyright (c) 2005, Regents of the University of California
 * All rights reserved.
 *
 * Author: Adam Chlipala
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name of the University of California, Berkeley nor the names of
 *   its contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include "intlist.h"

BEGIN_FUNC(100);
STACK_TYPE(4, CUSTOM(LIST(INT)));
REG_TYPE(EAX, CUSTOM(LIST(INT)));
END_FUNC();
list *int_list_id(list *ls) {
  return ls;
}

BEGIN_FUNC(100);
STACK_TYPE(4, CUSTOM(LIST(INT)));
END_FUNC();
int int_list_hd(list *ls) {
  int ret;

  if (ls)
    ret = (int)ls->data;
  else {
    BEGIN_CUTPOINT(50);
    REG_TYPE(ESP, STACK(16));
    REG_TYPE(EBP, STACK(12));
    STACK_TYPE(8, RETPTR);
    STACK_TYPE(12, OLDEBP);
    END_CUTPOINT();
    ret = -1;
  }

  BEGIN_CUTPOINT(50);
  REG_TYPE(ESP, STACK(16));
  REG_TYPE(EBP, STACK(12));
  STACK_TYPE(8, RETPTR);
  STACK_TYPE(12, OLDEBP);
  END_CUTPOINT();
  return ret;
}

BEGIN_FUNC(100);
STACK_TYPE(4, CUSTOM(LIST(INT)));
REG_TYPE(EAX, CUSTOM(LIST(INT)));
END_FUNC();
list *int_list_tl(list *ls) {
  list *ret;

  if (ls)
    ret = ls->next;
  else {
    BEGIN_CUTPOINT(50);
    REG_TYPE(ESP, STACK(16));
    REG_TYPE(EBP, STACK(12));
    STACK_TYPE(8, RETPTR);
    STACK_TYPE(12, OLDEBP);
    END_CUTPOINT();
    ret = 0;
  }

  BEGIN_CUTPOINT(50);
  REG_TYPE(ESP, STACK(16));
  REG_TYPE(EBP, STACK(12));
  STACK_TYPE(8, RETPTR);
  STACK_TYPE(12, OLDEBP);
  STACK_TYPE(16, CUSTOM(LIST(INT)));
  END_CUTPOINT();
  return ret;
}

BEGIN_FUNC(100);
STACK_TYPE(8, CUSTOM(LIST(INT)));
STACK_TYPE(4, CUSTOM(INT));
END_FUNC();
void int_list_set_hd(list *ls, void *d) {
  if (ls)
    ls->data = d;

  BEGIN_CUTPOINT(50);
  REG_TYPE(ESP, STACK(16));
  REG_TYPE(EBP, STACK(16));
  STACK_TYPE(12, RETPTR);
  STACK_TYPE(16, OLDEBP);
  END_CUTPOINT();
}

BEGIN_FUNC(100);
STACK_TYPE(8, CUSTOM(LIST(INT)));
STACK_TYPE(4, CUSTOM(LIST(INT)));
END_FUNC();
void int_list_set_tl(list *ls, list *d) {
  if (ls)
    ls->next = d;

  BEGIN_CUTPOINT(50);
  REG_TYPE(ESP, STACK(16));
  REG_TYPE(EBP, STACK(16));
  STACK_TYPE(12, RETPTR);
  STACK_TYPE(16, OLDEBP);
  END_CUTPOINT();
}

BEGIN_FUNC(70);
STACK_TYPE(4, CUSTOM(LIST(INT)));
END_FUNC();
int int_list_len(list *ls) {
  int len = 0;

  BEGIN_CUTPOINT(50);
  REG_TYPE(ESP, STACK(16));
  REG_TYPE(EBP, STACK(12));
  STACK_TYPE(4, CUSTOM(LIST(INT)));
  STACK_TYPE(8, RETPTR);
  STACK_TYPE(12, OLDEBP);
  END_CUTPOINT();
  for (;;) {
    if (ls) {
      ++len;
      ls = ls->next;
    } else {
      BEGIN_CUTPOINT(50);
      REG_TYPE(ESP, STACK(16));
      REG_TYPE(EBP, STACK(12));
      STACK_TYPE(8, RETPTR);
      STACK_TYPE(12, OLDEBP);
      END_CUTPOINT();
      return len;
    }
  }
}

BEGIN_FUNC(100);
STACK_TYPE(4, CUSTOM(LIST(LIST(INT))));
END_FUNC();
int int_int_list_len(list *ls) {
  int len = 0;

  BEGIN_CUTPOINT(90);
  REG_TYPE(ESP, STACK(16));
  REG_TYPE(EBP, STACK(12));
  STACK_TYPE(4, CUSTOM(LIST(LIST(INT))));
  STACK_TYPE(8, RETPTR);
  STACK_TYPE(12, OLDEBP);
  END_CUTPOINT();
  for (;;) {
    if (ls) {
      len += int_list_len((list *)ls->data);
      ls = ls->next;
    } else {
      BEGIN_CUTPOINT(90);
      REG_TYPE(ESP, STACK(16));
      REG_TYPE(EBP, STACK(12));
      STACK_TYPE(8, RETPTR);
      STACK_TYPE(12, OLDEBP);
      END_CUTPOINT();
      return len;
    }
  }
}

BEGIN_FUNC(70);
STACK_TYPE(4, CUSTOM(LIST(INT)));
REG_TYPE(EAX, CUSTOM(LIST(INT)));
END_FUNC();
list *int_list_rev(list *ls) {
  list *tmp, *prev = 0;

  BEGIN_CUTPOINT(50);
  REG_TYPE(ESP, STACK(20));
  REG_TYPE(EBP, STACK(12));
  STACK_TYPE(4, CUSTOM(LIST(INT)));
  STACK_TYPE(8, RETPTR);
  STACK_TYPE(12, OLDEBP);
  STACK_TYPE(16, CUSTOM(LIST(INT)));
  END_CUTPOINT();
  for (;;) {
    if (ls) {
      tmp = ls->next;
      ls->next = prev;
      ls = tmp;
    } else {
      BEGIN_CUTPOINT(50);
      REG_TYPE(ESP, STACK(20));
      REG_TYPE(EBP, STACK(12));
      STACK_TYPE(8, RETPTR);
      STACK_TYPE(12, OLDEBP);
      STACK_TYPE(16, CUSTOM(LIST(INT)));
      END_CUTPOINT();
      return prev;
    }
  }
}

BEGIN_FUNC(8186);
END_FUNC();
int main() {
  return 0;
}
